@@ -301,10 +301,10 @@ class Form(object):
         renderer = FormRenderer(self, self._translate)
         form = renderer.render(values=values, buttons=buttons,
                                previous_values=previous_values)
-        try:
-            return htmlfill.render(form, values)
-        except AssertionError, e:
-            log.error("Error while parsing form in htmlfill: %s. Hint See 'https://github.com/formencode/formencode/issues/57'" % e.message)
+        # TODO: Can happen: Error while parsing form in htmlfill: %s. Hint See
+        # 'https://github.com/formencode/formencode/issues/57desc (ti)
+        # <2014-03-21 10:29> 
+        return htmlfill.render(form, values)
 
     def _add_error(self, fieldname, error):
         field = self.get_field(fieldname)
@@ -626,7 +626,19 @@ class Field(object):
         self.renderer = get_renderer(self, translate)
         self._errors = []
         self._warnings = []
-        self.value = ""
+        # Set default value
+        value = getattr(self._config, "value")
+        if value and value.startswith("$"):
+            # value is a field. Try to get the value of the field.
+            try:
+                value = getattr(self._form._item, value.strip("$"))
+            except IndexError, e:
+                log.error("Error while accessing attribute '%s': %s" % (value, e))
+                value = None
+            except AttributeError, e:
+                log.error("Error while accessing attribute '%s': %s" % (value, e))
+                value = None
+        self.value = value
         self.previous_value = None
         """Value as string of the field. Will be set on rendering the
         form"""
@@ -708,12 +720,38 @@ class Field(object):
     def _build_filter_expr(self, expr_str, item):
         t = expr_str.split(" ")
         for x in t:
+            # % marks attributes of the current fields item in case of
+            # selections. Is used to iterate over the items in the selection.
             if x.startswith("%"):
                 key =  x.strip("%")
                 value = getattr(item, key)
+            # @ marks the item of the current fields form item.
+            elif x.startswith("@"):
+                key =  x.strip("@")
+                value = getattr(self._form._item, key)
+            # $ special attributes of the current form.
+            elif x.startswith("$"):
+                tmpitem = None
+                value = None
+                tokens = x.split(".")
+                if len(tokens) > 1:
+                    key = tokens[0].strip("$")
+                    attribute = ".".join(tokens[1:])
+                    if key == "user":
+                        tmpitem = self._form._request.user
+                if tmpitem and not value:
+                    value = getattr(tmpitem, attribute)
+                    if hasattr(value, '__call__'):
+                        value = value()
+            else:
+                value = None
+
+            if value:
                 if isinstance(value, list):
                     value = "[%s]" % ", ".join("'%s'" % unicode(v) for v in value)
                     expr_str = expr_str.replace(x, value)
+                else:
+                    expr_str = expr_str.replace(x, "'%s'" % str(value))
         p = Parser()
         return p.parse(expr_str)
 
