@@ -1,4 +1,5 @@
 import logging
+import difflib
 
 from mako.lookup import TemplateLookup
 from formbar import template_dir
@@ -31,12 +32,16 @@ def get_renderer(field, translate):
             return InfoFieldRenderer(field, translate)
         elif renderer.render_type == "dropdown":
             return DropdownFieldRenderer(field, translate)
+        elif renderer.render_type == "selection":
+            return SelectionFieldRenderer(field, translate)
         elif renderer.render_type == "datepicker":
             return DateFieldRenderer(field, translate)
         elif renderer.render_type == "password":
             return PasswordFieldRenderer(field, translate)
         elif renderer.render_type == "hidden":
             return HiddenFieldRenderer(field, translate)
+        elif renderer.render_type == "html":
+            return HTMLRenderer(field, translate)
     else:
         # Try to determine the datatype of the field and set approriate
         # renderer.
@@ -84,19 +89,24 @@ class FormRenderer(Renderer):
         self.translate = translate
         self.template = template_lookup.get_template("form.mako")
 
-    def render(self, buttons=True, values=None):
+    def render(self, buttons=True, values=None, previous_values=None):
         """Returns the rendered form as string.
 
         :buttons: Boolean flag to indicate if the form buttons should be
         rendererd. Defaults to true.
+        :previous_values: Dictionary of values of the last saved state
+        of the item. If provided a diff between the current and previous
+        values will be renderered in readonly mode.
         :returns: rendered form.
 
         """
         if not values:
             values = {}
+        if not previous_values:
+            previous_values = {}
         html = []
         html.append(self._render_form_start())
-        html.append(self._render_form_body(values))
+        html.append(self._render_form_body(values, previous_values))
         if not self._form._config.readonly and buttons:
             html.append(self._render_form_buttons())
         html.append(self._render_form_end())
@@ -121,9 +131,10 @@ class FormRenderer(Renderer):
                         % self._form._csrf_token)
         return "".join(html)
 
-    def _render_form_body(self, values):
+    def _render_form_body(self, values, previous_values):
         values = {'form': self._form,
                   '_': self.translate,
+                  'previous_values': previous_values,
                   'values': values}
         return self.template.render(**values)
 
@@ -187,8 +198,55 @@ class FieldRenderer(Renderer):
         values = self._get_template_values()
         return template.render(**values)
 
+    def _render_diff(self, newvalue, oldvalue):
+        """Will return a HTML string showing the differences between the old and
+        the new string.
+
+        The new string will have some markup to show the differences between
+        the given strings. Words which has been deleted in the new string
+        are marked with a *span* tag with the class *formed-deleted-value*.
+        Elements which are new in the new string are marked with a span tag
+        having the class *formed-new-value*.
+
+        :old: Old string
+        :new: New string
+        :returns: A HTML string showing the differences.
+
+        """
+        out = []
+        mode = None
+        d = difflib.Differ()
+        old = newvalue.split(" ")
+        new = oldvalue.split(" ")
+        diff = d.compare(old, new)
+        for x in diff:
+            if x[0:2] == "+ " and mode != "new":
+                if mode:
+                    out.append("</span>")
+                    mode = None
+                out.append('<span class="formbar-new-value">')
+                mode = "new"
+            elif x[0:2] == "- " and mode != "del":
+                if mode:
+                    out.append("</span>")
+                    mode = None
+                out.append('<span class="formbar-del-value">')
+                mode = "del"
+            elif x[0:2] == "  ":
+                if mode:
+                    out.append("</span>")
+                    mode = None
+            elif x[0:2] == "? ":
+                continue
+            out.append("%s " % "".join(x[2::]))
+        if mode:
+            out.append("</span>")
+        return "".join(out)
+
+
     def _get_template_values(self):
         values = {'field': self._field,
+                  'renderer': self,
                   '_': self.translate}
         return values
 
@@ -274,6 +332,20 @@ class HiddenFieldRenderer(FieldRenderer):
         html.append(self.template.render(**values))
         return "".join(html)
 
+class HTMLRenderer(FieldRenderer):
+    """A Renderer to render generic HTML"""
+
+    def __init__(self, field, translate):
+        FieldRenderer.__init__(self, field, translate)
+        self.template = template_lookup.get_template("html.mako")
+
+    def render(self):
+        html = []
+        values = self._get_template_values()
+        html.append(self._render_label())
+        html.append(self.template.render(**values))
+        return "".join(html)
+
 
 # TODO: Use a new superclass for the following two renderers as they
 # deal not only with one single value but with multiple selectable
@@ -286,11 +358,14 @@ class DropdownFieldRenderer(FieldRenderer):
     def __init__(self, field, translate):
         FieldRenderer.__init__(self, field, translate)
         self.template = template_lookup.get_template("dropdown.mako")
+        self._cache_options = None
 
     def _get_template_values(self):
         values = FieldRenderer._get_template_values(self)
         # Add the options to the values dictionary
-        values['options'] = self._field.get_options()
+        if self._cache_options is None:
+            self._cache_options = self._field.get_options()
+        values['options'] = self._cache_options
         return values
 
 class SelectionFieldRenderer(FieldRenderer):
